# Sherlock version 1.0.0
# Ramsey.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of Sherlock, a primitive solver for numerical contraint satisfaction problems.
# Sherlock is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Sherlock is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Sherlock.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Script
# 3) Main

# 1) Imported modules
# -------------------

import string
import sys
import re

# 2) Script
# ---------

def Ramsey_PrintHeader(output):
  output.write("""/*
 * Ramsey problem
 */

domain Color [1, 3]

""")

def Ramsey_PrintProblem(numberOfNodes, output):
  output.write("problem Ramsey%d\n" % numberOfNodes)
  for s in range(1, numberOfNodes):
    output.write("  Color ");
    for t in range(s+1, numberOfNodes+1):
      if t>s+1:
        output.write(", ");
      output.write("C%d_%d" % (s, t));
    output.write(";\n");
  output.write("\n");
  output.write("  constraint\n");
  for i in range(1, numberOfNodes-1):
    for j in range(i+1, numberOfNodes):
      for k in range(j+1, numberOfNodes+1):
        for c in range(1, 4):
          output.write("    or C%d_%d!=%d, C%d_%d!=%d, C%d_%d!=%d;\n" % (i, j, c, i, k, c, j, k, c));
  output.write("end\n");

# 3) Main
# -------

fileName = "Ramsey.csp"
try:
  file = open(fileName, "w")
except:
  sys.stderr.write('Unable to open file "%s"\n' % fileName)
  sys.stderr.flush()
  sys.exit(1)

Ramsey_PrintHeader(file);
file.write("\n")
Ramsey_PrintProblem(16, file)
file.write("\n")
Ramsey_PrintProblem(17, file)
file.close()
